#include "stdafx.h"
#include "Parser.h"

SimpleHtmlParser::SimpleHtmlParser()
{
	m_nNodes = NULL;
}

SimpleHtmlParser::~SimpleHtmlParser()
{
	if( m_nNodes != NULL ) ReleaseNodes();
}

bool SimpleHtmlParser::_isblank(char ch)
{
	if( ch == ' ' || ch == '\t' || ch == '\r' || ch == '\n')
		return true;
	return false;
}

SimpleHtmlParser::node_flag SimpleHtmlParser::_parser_flag(const std::string& tag)
{
	node_flag nf;
	if( _stricmp(tag.c_str() , "p") == 0) nf = nf_pre;
	else if( _stricmp(tag.c_str() , "b") == 0) nf = nf_bold;
	else if( _stricmp(tag.c_str() , "l") == 0) nf = nf_italics;
	else if( _stricmp(tag.c_str() , "u") == 0) nf = nf_undeline;
	else if( _stricmp(tag.c_str() , "ul") == 0) nf = nf_ul;
	else if( _stricmp(tag.c_str() , "li") == 0) nf = nf_li;
	else nf = nf_none;
	return nf;
}

SimpleHtmlParser::html_node* SimpleHtmlParser::_make_node()
{
	html_node* p = new html_node;
	return p;
}

SimpleHtmlParser::html_node* SimpleHtmlParser::GetNodes()
{
	return m_nNodes;
}

std::string SimpleHtmlParser::GetSubText(SimpleHtmlParser::html_node* node)
{
	std::string strText;
	html_node* cur = node;
	while( cur != NULL)
	{
		strText += cur->content;
		cur = cur->child;
	}
	return strText;
}

int SimpleHtmlParser::GetInheritAttributes(SimpleHtmlParser::html_node* node)
{
	int flag = nf_none;
	SimpleHtmlParser::html_node* cur = node;
	while(cur != NULL)
	{
		flag += cur->flag;
		cur = cur->parent;
	}
	return flag;
}

SimpleHtmlParser::html_node* SimpleHtmlParser::ParserHtml(const char *html)
{
	const char* cur = html;
	const char* end = NULL;
	const char* begin = NULL;

	bool in_node = false;
	html_status st = st_begin;
	if( m_nNodes != NULL ) ReleaseNodes();
	
	struct html_node* cur_node = m_nNodes;

	while(*cur != 0)
	{
		switch(st)
		{
		case st_begin:
			{
				if( cur[0] == '<' )
				{					
					st = st_tagstart;
				}
				++cur;
				break;
			}
		case st_tagstart:
			{	
				while(_isblank(cur[0]))
				{
					++cur;
				}

				st = st_tagend;
				if( cur[0] == '/' )
				{
					st = st_endtagbegin;
					++cur;
				}
				else
				{			
					begin = cur;
					end = NULL;
					++cur;
				}				
				break;
			}
		case st_tagend:
			{
				int tail = 0;
				while(_isblank(cur[0]))
				{
					++tail;
					++cur;
				}

				if( cur[0] == '>' )
				{
					end = cur - tail;
				}

				if( end != NULL )
				{
					st = st_contentbegin;
					std::string tag = std::string(begin , end - begin);
					if( cur_node == NULL )
					{
						cur_node = _make_node();
						m_nPointers.push_back(cur_node);
						cur_node->flag = _parser_flag(tag);
						if( cur_node->flag & nf_pre || cur_node->flag & nf_ul || cur_node->flag & nf_li)
							cur_node->type = nt_block;
						else
							cur_node->type = nt_innder;
						if( m_nNodes == NULL ) m_nNodes = cur_node;
					}
					else
					{
						html_node* new_node = _make_node();	
						m_nPointers.push_back(new_node);
						if( cur_node->closed )
						{
							cur_node->next = new_node;
							cur_node = new_node;
							cur_node->flag += _parser_flag(tag);
							if( cur_node->flag & nf_pre || cur_node->flag & nf_ul || cur_node->flag & nf_li)
								cur_node->type = nt_block;
							else
								cur_node->type = nt_innder;
						}
						else
						{							
							new_node->parent = cur_node;

							struct html_node* tmp = cur_node;
							while(tmp->child != NULL)
							{
								tmp = tmp->child;
							}
							
							tmp->child = new_node;
							cur_node = new_node;
							cur_node->flag = _parser_flag(tag);
							if( cur_node->flag & nf_pre || cur_node->flag & nf_ul || cur_node->flag & nf_li)
								cur_node->type = nt_block;
							else
								cur_node->type = nt_innder;
						}
					}
				}

				++cur;
				break;
			}
		case st_contentbegin:
			{
				begin = cur;
				st = st_contentend;
				end = NULL;				
				if(cur[0] != '<' ) ++cur;
				break;
			}
		case st_contentend:
			{
				if( cur[0] == '<' )
				{
					end = cur;
					st = st_tagstart;
					for( const char* index = begin ; index < end ; ++index )
					{
						if( *index != '\r' && *index != '\n' ) cur_node->content += *index;
					}
				}

				++cur;
				break;
			}
		case st_endtagbegin:
			{
				while(_isblank(cur[0]))
				{
					++cur;
				}

				begin = cur;
				end = NULL;
				st = st_endtagend;
				++cur;
				break;
			}
		case st_endtagend:
			{
				int tail = 0;
				while(_isblank(cur[0]))
				{
					++cur;
					++tail;
				}

				if( cur[0] == '>' )
				{
					end = cur - tail;
				}

				++cur;				
				if( end != NULL )
				{
					st = st_contentbegin;
					cur_node->closed = true;
					std::string tag = std::string(begin , end - begin);

					if( cur_node->parent != NULL )
					{
						cur_node = cur_node->parent;
					}
				}
				break;
			}
		default:
			break;
		}
	}

	return m_nNodes;
}

void SimpleHtmlParser::ReleaseNodes()
{
	for( std::vector<struct html_node*>::iterator it = m_nPointers.begin() ; it != m_nPointers.end() ; ++it )
	{
		delete *it;
	}

	m_nPointers.clear();
	m_nNodes = NULL;
}

int SimpleCsvParser::SplitString(const std::string &strContent, StringArray &nValues, const std::string &strKey, int nMaxItems, bool bIgnoreEmpty)
{
	int nStartPos = 0;
	int nFindPos = 0;
	int nItemCount = 0;
	int nKeyLen = (int)strKey.size();
	int nEndPos = (int)strContent.size();

	nValues.clear();

	if( strKey.size() == 0 ) return 0;
	else if( nEndPos <= 0 || nEndPos >= (int)strContent.size() )
	{
		nEndPos = (int)strContent.size();
	}

	while( nStartPos < (int)nEndPos )
	{
		nFindPos = (int)strContent.find(strKey.c_str(), nStartPos);
		if( nFindPos < 0 || nFindPos >= nEndPos || (nMaxItems > 0 && nItemCount == nMaxItems - 1) )
		{
			nFindPos = nEndPos;
		}

		if( nFindPos < nStartPos )
		{
			break;
		}

		if( nFindPos > nStartPos || !bIgnoreEmpty )
		{
			std::string strNewValue = (nFindPos > nStartPos) ? strContent.substr(nStartPos, nFindPos - nStartPos) : std::string("");
			nValues.push_back(strNewValue);
			nItemCount ++;
		}

		nStartPos = nFindPos + nKeyLen;
	}

	return nItemCount;
}

void SimpleCsvParser::ParseCvsText(const char *text)
{
	m_nValues.clear();
	m_nMaxValue = 0;	
	m_nColHeader.clear();
	m_nRowHeader.clear();	
	m_nColCount = m_nRowCount = 0;

	StringArray nLineArray;
	std::string strContent = text;
	SplitString(strContent , nLineArray , "\r\n" , -1 , false);
	for( StringArray::iterator it = nLineArray.begin() ; it != nLineArray.end() ; ++it )
	{
		StringArray nColArray;
		SplitString(*it , nColArray , "\t" , -1 , false);
		if( m_nColHeader.empty() ) 
		{
			for( StringArray::iterator itII = ++nColArray.begin() ; itII != nColArray.end() ; ++itII)
				m_nColHeader.push_back(*itII);
		}
		else
		{
			for(StringArray::iterator itII = nColArray.begin() ; itII != nColArray.end() ; ++itII)
			{
				if( itII == nColArray.begin() )
				{
					m_nRowHeader.push_back(*itII);
				}
				else
				{
					int value = atoi(itII->c_str());
					if( value > m_nMaxValue ) m_nMaxValue = value;
					m_nValues.push_back(value);
				}
			}
		}
	}

	m_nColCount = m_nColHeader.size();
	m_nRowCount = m_nRowHeader.size();
// 	for( StringArray::iterator it = m_nRowHeader.begin() ; it != m_nRowHeader.end() ; ++it )
// 	{
// 		HBRUSH hBrush = ::CreateSolidBrush(RGB()
// 	}
}

int SimpleCsvParser::GetMaxValue()
{
	return m_nMaxValue;
}

std::vector<int> SimpleCsvParser::GetRowValue(const std::string& strRow)
{
	size_t index = 0 ;
	for( StringArray::iterator it = m_nRowHeader.begin() ; it != m_nRowHeader.end() ; ++it )
	{
		if( *it == strRow )
			break;
		else
			++index;
	}

	std::vector<int> values;
	for( size_t i = m_nColCount * index ; i < m_nColCount * (index+1) ; ++i )
	{
		values.push_back(m_nValues.at(i));
	}

	return values;
}

std::vector<int> SimpleCsvParser::GetColValue(const std::string& strCol)
{
	size_t index = 0;
	for( StringArray::iterator it = m_nColHeader.begin() ; it != m_nColHeader.end() ; ++it )
	{
		if(*it == strCol)
			break;
		else
			++index;
	}

	std::vector<int> values;
	while(  index < m_nValues.size() )
	{
		values.push_back( m_nValues.at(index) );
		index += m_nColCount;
	}
	
	return values;
}