#ifndef __PARSER_H_INCLUDE__
#define __PARSER_H_INCLUDE__

#include <map>
#include <string>
#include <vector>

typedef std::vector<std::string> StringArray;

class SimpleCsvParser
{
public:
	int GetMaxValue();

	void ReleaseNode();

	void ParseCvsText(const char* text);

	StringArray GetColHeaders() { return m_nColHeader; }

	StringArray GetRowHeaders() { return m_nRowHeader; }	

	std::vector<int> GetRowValue(const std::string& strRow);

	std::vector<int> GetColValue(const std::string& strCol);

private:	

	int SplitString(const std::string &strContent, StringArray &nValues, const std::string &strKey, int nMaxItems, bool bIgnoreEmpty);

private:
	int m_nMaxValue;
	int m_nRowCount;
	int m_nColCount;	
	StringArray m_nColHeader;
	StringArray m_nRowHeader;	
	std::vector<int> m_nValues;
	std::map<std::string , HBRUSH> m_nRowColors;
};

class SimpleHtmlParser
{
public:
	enum node_flag
	{
		nf_none = 0,
		nf_pre = 1,
		nf_bold = 2,
		nf_italics = 4,
		nf_undeline = 8 , 
		nf_ul = 16,
		nf_li = 32
	};

	enum node_type
	{
		nt_block,
		nt_innder
	};

	struct html_node
	{
		int flag;
		int type;
		bool closed;
		std::string content;
		struct html_node* next;
		struct html_node* child;
		struct html_node* parent;

		html_node()
		{
			closed = false;
			type = nt_block;
			flag = nf_none;		
			next = child = parent = NULL;
		}
	};

	SimpleHtmlParser();
	~SimpleHtmlParser();

public:
	void ReleaseNodes();

	SimpleHtmlParser::html_node* GetNodes();

	std::string GetSubText(SimpleHtmlParser::html_node* node);

	int GetInheritAttributes(SimpleHtmlParser::html_node* node);

	SimpleHtmlParser::html_node* ParserHtml(const char* html);

private:
	bool _isblank(char ch);	
	struct html_node* _make_node();	
	SimpleHtmlParser::node_flag _parser_flag(const std::string& tag);

private:
	enum html_status
	{
		st_begin , 
		st_tagstart,
		st_tagend,
		st_contentbegin,
		st_contentend,
		st_endtagbegin,
		st_endtagend,
		st_end
	};
	struct html_node* m_nNodes;
	std::vector<struct html_node*> m_nPointers;
};



#endif
